
import hbp_nrp_cle.tf_framework as nrp
from hbp_nrp_cle.robotsim.RobotInterface import Topic
from gazebo_msgs.srv import SetModelState
import rospy

rospy.wait_for_service("/gazebo/set_model_state")
service_proxy = rospy.ServiceProxy('/gazebo/set_model_state', SetModelState, persistent=True)


@nrp.MapSpikeSink("leftNeuronRec", nrp.brain.leftNeuron, nrp.spike_recorder)
@nrp.MapRobotSubscriber("position", Topic('/gazebo/model_states', gazebo_msgs.msg.ModelStates))
@nrp.MapVariable("setModelStateSrv", initial_value=service_proxy)
@nrp.Neuron2Robot(triggers="leftNeuronRec")
def brainToRobotLeft(t, leftNeuronRec,position,setModelStateSrv):
        if (len(leftNeuronRec.times) == 0):
                return

        #get the current robot state.
        RobotIndexValue = position.value.name.index('robot')
        posX = position.value.pose[RobotIndexValue].position.x
        posY = position.value.pose[RobotIndexValue].position.y
        zOrientation = position.value.pose[RobotIndexValue].orientation.z
        wOrientation = position.value.pose[RobotIndexValue].orientation.w
    
        #set up the message to change the robot
        ms_msg =  gazebo_msgs.msg.ModelState()
        ms_msg.model_name = 'robot'

        #set up the variables for the movement.
        #get the states from the brain to move the robot
        clientLogger.info("L Spikes recorded: {}".format(leftNeuronRec.times))

        #set orientation to the clock direction.
        if ((zOrientation < .75)  and (zOrientation > .65) and (wOrientation < .75)  and (wOrientation > .65)):
                orientation = 3
        elif ((zOrientation < 1.1)  and (zOrientation > .9) and (wOrientation < .1)  and (wOrientation > -.1)):
                orientation = 12
        elif ((zOrientation < -.65)  and (zOrientation > -.75) and (wOrientation < .75)  and (wOrientation > .65)):
                orientation = 9
        else:
                orientation = 6
                
        if (orientation == 3):
                newOrientation = 12
        elif (orientation == 12):
                newOrientation = 9
        elif (orientation == 9):
                newOrientation = 6
        else:
                newOrientation = 3

        #translate newOrientation clock orientation to w and z
        if (newOrientation == 3):
                newZOrientation = .7
                newWOrientation = .7
        elif (newOrientation == 12):
                newZOrientation = 1
                newWOrientation = 0
        elif (newOrientation == 9):
                newZOrientation = -.7
                newWOrientation = .7
        else: #6
                newZOrientation = -1
                newWOrientation = 0

        #fill the msg
        ms_msg.pose.position.x = posX
        ms_msg.pose.position.y = posY
        ms_msg.pose.position.z = 0.0
        #orientation
        ms_msg.pose.orientation.x = 0.0
        ms_msg.pose.orientation.y = 0.0
        ms_msg.pose.orientation.z =  newZOrientation
        ms_msg.pose.orientation.w =  newWOrientation
        #scale
        ms_msg.scale.x = ms_msg.scale.y = ms_msg.scale.z = 1.0
        #send the message
        response = setModelStateSrv.value(ms_msg)

