# This gets the brain state from neurons 10 and 30.  If these are relatively
#active, it turns or moves the robot.
#The signal neurons start, or move the four state brain model.
#Position gets the state of the robot in the environment.
#The robot is changed by a message sent in this file.
#The return value is vacuous.
import hbp_nrp_cle.tf_framework as nrp
from hbp_nrp_cle.robotsim.RobotInterface import Topic
from gazebo_msgs.srv import SetModelState
import rospy
import geometry_msgs.msg

rospy.wait_for_service("/gazebo/set_model_state")
service_proxy = rospy.ServiceProxy('/gazebo/set_model_state', SetModelState, persistent=True)

@nrp.MapSpikeSink("forwardNeuron", nrp.brain.neurons[10], nrp.leaky_integrator_alpha)
@nrp.MapSpikeSink("rightNeuron", nrp.brain.neurons[30], nrp.leaky_integrator_alpha)
@nrp.MapSpikeSource("startStateSignalNeuron", nrp.brain.neurons[0], nrp.fixed_frequency,weight=0.12,initial_rate=10)
@nrp.MapSpikeSource("switchStateSignalNeuron0", nrp.brain.neurons[2], nrp.fixed_frequency,weight=0.12)
@nrp.MapSpikeSource("switchStateSignalNeuron1", nrp.brain.neurons[5], nrp.fixed_frequency,weight=0.12)
@nrp.MapRobotSubscriber("position", Topic('/gazebo/model_states', gazebo_msgs.msg.ModelStates))
@nrp.MapVariable("set_model_state_srv", initial_value=service_proxy)
@nrp.Neuron2Robot(Topic('/husky/cmd_vel', geometry_msgs.msg.Twist))

def moveRobotInSquare(t, startStateSignalNeuron,switchStateSignalNeuron0,switchStateSignalNeuron1,rightNeuron,forwardNeuron,position,set_model_state_srv):

    #This is the hz that are sent to the brain state changes.  The higher
    #the faster the state changes.
    brainStateChangeRate = 10
    
    #Set the signal to the brain so that starts then moves through its 4 states.
    if (startStateSignalNeuron.rate > 0):
        rate = startStateSignalNeuron.rate
        rate = rate -1
        startStateSignalNeuron.rate = rate
    elif (switchStateSignalNeuron0.rate == 0):
        startStateSignalNeuron.rate = 0
        switchStateSignalNeuron0.rate = brainStateChangeRate
        switchStateSignalNeuron1.rate = 0
    else:
        startStateSignalNeuron.rate = 0
        switchStateSignalNeuron0.rate = 0
        switchStateSignalNeuron1.rate = brainStateChangeRate

    #get the states from the brain to move the robot
    voltageRight=rightNeuron.voltage
    voltageForward=forwardNeuron.voltage
    
    #print some stuff
    f = open ("/home/chris/tempN2R.txt","a")
    f.write(str(voltageRight))
    f.write(" ")
    f.write(str(voltageForward))
    f.write("line\n")

    #set up the variables for the movement.
    forward = 0
    turn = 0

    if (voltageRight > 0.02):
        turn = 1
        f.write("rob right\n")
    if (voltageForward > 0.02):
        forward = 1
        f.write("rob forward\n")

    #get the current robot state.
    RobotIndexValue = position.value.name.index('robot')
    posX = position.value.pose[RobotIndexValue].position.x
    posY = position.value.pose[RobotIndexValue].position.y
    zOrientation = position.value.pose[RobotIndexValue].orientation.z
    wOrientation = position.value.pose[RobotIndexValue].orientation.w
    
    f.write(" ")
    f.write(str(posX))
    f.write(" ")
    f.write(str(posY))
    f.write(" ")
    f.write(str(zOrientation))
    f.write(" ")
    f.write(str(wOrientation))

    #set orientation to the clock direction.
    if ((zOrientation < .75)  and (zOrientation > .65) and (wOrientation < .75)  and (wOrientation > .65)):
        orientation = 3
    elif ((zOrientation < 1.1)  and (zOrientation > .9) and (wOrientation < .1)  and (wOrientation > -.1)):
        orientation = 12
    elif ((zOrientation < -.65)  and (zOrientation > -.75) and (wOrientation < .75)  and (wOrientation > .65)):
        orientation = 9
    else:
        orientation = 6

    f.write(" ")
    f.write(str(orientation))
    
    #if it's turning, change to the next of 4 orientations
    if (turn == 1):
        if (orientation == 3):
            newOrientation = 12
        elif (orientation == 12):
            newOrientation = 9
        elif (orientation == 9):
            newOrientation = 6
        else:
            newOrientation = 3
    else:
        newOrientation = orientation

    #translate newOrientation value to w and z
    if (newOrientation == 3):
        newZOrientation = .7
        newWOrientation = .7
    elif (newOrientation == 12):
        newZOrientation = 1
        newWOrientation = 0
    elif (newOrientation == 9):
        newZOrientation = -.7
        newWOrientation = .7
    else: #6
        newZOrientation = -1
        newWOrientation = 0

    #set up the message to change the robot
    ms_msg =  gazebo_msgs.msg.ModelState()
    ms_msg.model_name = 'robot'
    if (forward == 1):
        if (newOrientation == 3):
            posY  = 0.1 + posY
        elif (newOrientation == 12):
            posX  = -0.1 + posX
        elif (newOrientation == 9):
            posY  = -0.1 + posY
        else:
            posX  = 0.1 + posX
        
    ms_msg.pose.position.x = posX
    ms_msg.pose.position.y = posY
    ms_msg.pose.position.z = 0.0
    #orientation
    ms_msg.pose.orientation.x = 0.0
    ms_msg.pose.orientation.y = 0.0
    ms_msg.pose.orientation.z =  newZOrientation
    ms_msg.pose.orientation.w =  newWOrientation
    #scale
    ms_msg.scale.x = ms_msg.scale.y = ms_msg.scale.z = 1.0
    #send the message
    response = set_model_state_srv.value(ms_msg)

    if not response.success:
        f.write(response.status_message)
    f.write(" x y zOrient wOrient clock \n")
    f.close()

    #the return is vacuous, this can be changed.
    return geometry_msgs.msg.Twist(linear=geometry_msgs.msg.Vector3(0,0,0),
                                   angular=geometry_msgs.msg.Vector3(0,0,0))


